//*----------------------------------------------------------------------------
//*
//* AT91SAM7 efsl example (9/2006)
//* by Martin Thomas, Kaiserslautern, Germany 
//* <mthomas@rhrk.uni-kl.de>
//* http://www.siwawi.arub.uni-kl.de/avr_projects
//*
//*----------------------------------------------------------------------------

#include <string.h>

#include "board_ex256.h"
#include "common/swi.h"
#include "systime.h"
#include "dbgu.h"

#include "inc/efs.h"
#include "inc/ls.h"
#include "inc/mkfs.h"
#include "inc/interfaces/efsl_dbg_printf_arm.h"

#define TCK  1000                           /* Timer Clock  */
#define PIV  ((MCK/TCK/16)-1)               /* Periodic Interval Value */

EmbeddedFileSystem efs;
EmbeddedFile filer, filew;
DirList list;
unsigned short e;
unsigned char buf[513];

static char LogFileName[] = "logSAM_7.txt";



signed char initcard(void)
{
	signed char res, retval;
	
	iprintf("CARD init...");
	if ( ( res = efs_init( &efs, 0 ) ) != 0 ) {
		iprintf("failed with %i\n",res);
		retval = -1;
	}
	else {
//		led1(1);
		iprintf("ok\n");
		fs_umount( &efs.myFs ) ;
//		led1(0);
		retval = 0;
	}
	return retval;
}

static void listrootdir(void)
{
	signed char res;
	
	iprintf("CARD init...");
	if ( ( res = efs_init( &efs, 0 ) ) != 0 ) {
		iprintf("failed with %i\n",res);
	}
	else {
//		led1(1);
		iprintf("ok\n");
		iprintf("\nDirectory of 'root':\n");
		ls_openDir( &list, &(efs.myFs) , "/");
		while ( ls_getNext( &list ) == 0 ) {
			list.currentEntry.FileName[LIST_MAXLENFILENAME-1] = '\0';
			iprintf( "%s ( %li bytes )\n" ,
				list.currentEntry.FileName,
				list.currentEntry.FileSize ) ;
		}
		fs_umount( &efs.myFs ) ;
//		led1(0);
	}
}

static void listfilecontent(void)
{
	signed char res;
	
	iprintf("CARD init...");
	if ( ( res = efs_init( &efs, 0 ) ) != 0 ) {
		iprintf("failed with %i\n",res);
	}
	else {
//		led1(1);
		iprintf("ok\n");
		if ( file_fopen( &filer, &efs.myFs , LogFileName , 'r' ) == 0 ) {
			iprintf("\nFile %s open. Content:\n", LogFileName);
			while ( ( e = file_read( &filer, 512, buf ) ) != 0 ) {
				buf[e]='\0';
				AT91F_DBGU_Printk((char*)buf);
			}
			iprintf("\n");
			file_fclose( &filer );
		}
		fs_umount( &efs.myFs ) ;
//		led1(0);
	}
}

static void append(void)
{
	signed char res;
	
	iprintf("CARD init...");
	if ( ( res = efs_init( &efs, 0 ) ) != 0 ) {
		iprintf("failed with %i\n",res);
	}
	else {
//		led1(1);
		iprintf("ok\n");
		if ( file_fopen( &filew, &efs.myFs , LogFileName , 'a' ) == 0 ) {
			iprintf("\nFile %s open for append. Appending...", LogFileName);
			strcpy((char*)buf, "Martin hat's angehaengt\r\n");
			if ( file_write( &filew, strlen((char*)buf), buf ) == strlen((char*)buf) ) {
				iprintf("ok\n");
			}
			else {
				iprintf("failed\n", LogFileName);
			}
			file_fclose( &filew );
		}
		fs_umount( &efs.myFs ) ;
//		led1(0);
	}
}


int main(void)
{
	int drawmenu;
	unsigned int choice;
	
	AT91PS_PMC  pPMC  = AT91C_BASE_PMC;
	AT91PS_PIO  pPIOA = AT91C_BASE_PIOA;
	AT91PS_RSTC pRSTC = AT91C_BASE_RSTC;
	
	// Enable the clock for PIO and UART0
	pPMC->PMC_PCER = ( ( 1 << AT91C_ID_PIOA ) | ( 1 << AT91C_ID_US0 ) ); // n.b. IDs are just bit-numbers
		
	// enable reset-key on demo-board 
	pRSTC->RSTC_RMR = (0xA5000000 | AT91C_RSTC_URSTEN);

	initcard();
	//systime_init();
	//AT91F_DBGU_Init();
	//IntEnable(); // enable INT-exceptions
	
	iprintf("\n\nAT91SAM7 Filesystem-Demo (P:AT91SAM7S L:efsl)\n"
	        "efsl AT91-Interface and this Demo-Application\n"
	        "done by Martin Thomas, Kaiserslautern, Germany\n\n");
	
	/* init efsl debug-output */
	//efsl_debug_devopen_arm(AT91F_DBGU_putc);
	
	drawmenu = 1;

	// main-loop
	while (1) {
		
		if ( drawmenu ) {
			iprintf("** Test Menu ** (testfile: %s)\n", LogFileName);
			iprintf("(1) Card-init\n");
			iprintf("(2) List root directory\n");
			iprintf("(3) List content of testfile\n");
			iprintf("(4) Append line to testfile\n");
			iprintf("(5) Benchmark (a simple one...)\n");
			iprintf("Select > ");
			drawmenu = 0;
		}
		else {
			iprintf("Select (0 shows menu) > ");
		}
		fflush(stdout);
		AT91F_DBGU_scanf("%i", &choice);
		iprintf("Selected %i\n", choice);

		switch (choice) {
		case 0 :
			drawmenu = 1;
			break;
		case 1 :
			initcard();
			break;
		case 2 :
			listrootdir();
			break;
		case 3 :
			listfilecontent();
			break;
		case 4 :
			append();
			break;
		case 5 :
			benchmark();
			break;
//		case 6 :
//			check_wpcp();
//			break;
//		case 7 : 
//			led1(1);
//			break;
//		case 8 :
//			led1(0);
//			break;
		default:
			iprintf("Invalid Choice\n");
			break;
		}
	}
	
	return 0; /* never reached */
}
